package main

import (
	"crypto/sha256"
	"database/sql"

	"bitbucket.org/prmunty/middleware/auth"
	migration "bitbucket.org/prmunty/util/dbmigration/db"
	sqlite "bitbucket.org/prmunty/util/dbmigration/sqliteutils"
	"bitbucket.org/prmunty/util/ethernet"
	_ "github.com/mattn/go-sqlite3"
)

func main() {
	const version float64 = 1.0

	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		return migration.DoUntilErr(tx,
			AddNewHiddenUsers,
			AddLocalHostConfigTables,
			FixFreezeScreenSelections,
		)
	})

}

func AddNewHiddenUsers(tx *sql.Tx) (err error) {
	return migration.DoUntilErr(tx,
		func(tx *sql.Tx) error { return addNewUser(tx, "Frick Service", hashPin("37378423"), auth.FrickService) },
		func(tx *sql.Tx) error { return addNewUser(tx, "Ringout", hashPin("7464688"), auth.Ringout) },
		func(tx *sql.Tx) error { return addNewUser(tx, "Controls", hashPin("26687657"), auth.Factory) },
	)

}

func addNewUser(tx *sql.Tx, firstName, pin string, secLevel auth.SecLevel) error {
	if _, err := tx.Exec("DELETE FROM users where secLevel = ?", secLevel); err != nil && err != sql.ErrNoRows {
		return err
	}

	_, err := tx.Exec(`INSERT INTO users (firstName, lastname, email, phone, carrier, pin,secLevel) 
             VALUES (?, "User", '', '','', ?, ?)`, firstName, pin, int(secLevel))
	return err

}

// hashPin hashes a Pin and returns the hashed value as a string
func hashPin(pin string) string {
	bytesArr := sha256.Sum256([]byte(pin))
	hashString := string(bytesArr[0:])
	return hashString
}

func AddLocalHostConfigTables(tx *sql.Tx) error {

	return migration.DoUntilErr(tx,
		addLocalHostConfig,
		addLocalHostComponents,
	)

}

func addLocalHostConfig(tx *sql.Tx) (err error) {
	if !(sqlite.DB{tx}.TableExists("local_host_configs")) {
		_, err = tx.Exec(`
                          CREATE TABLE local_host_configs(
                                 url TEXT,
                                 user_defined_name TEXT,
                                 product_type VARCHAR(25)
                          );
                          
                          INSERT INTO local_host_configs VALUES('', ?, 'compressor');`, getProductName(tx))
	}

	return err

}

func getProductName(tx *sql.Tx) (name string) {
	defaultName := "compressor"
	ip, err := ethernet.GetIP()
	if err != nil {
		return defaultName
	}

	err = tx.QueryRow("SELECT product_name FROM equipment WHERE ip = ? and type_name = ?", ip, "compressor").Scan(&name)
	if err != nil {
		return defaultName
	}

	return name

}

func addLocalHostComponents(tx *sql.Tx) (err error) {
	if !(sqlite.DB{tx}.TableExists("local_host_compnonents")) {
		_, err = tx.Exec(`
                          CREATE TABLE local_host_components(
                                 user_defined_name TEXT,
                                 product_type VARCHAR(25)
                          );`)

	}

	return err
}

func FixFreezeScreenSelections(tx *sql.Tx) error {
	freezeIDs := []int{12, 13, 14, 15, 16, 21, 27, 29, 132, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 739, 771, 772, 774, 775, 776, 778, 780, 781, 1082, 1083, 1084, 1085, 1179, 1512, 1513, 1531, 1565, 1566, 1567, 1568, 1569, 1596, 1666, 1743, 1744, 1745, 2153, 2162, 2174, 2195, 2212, 2213, 2214, 2215, 2219, 2227, 2228, 2320, 2321, 2542, 2923, 4244, 4245, 4246}

	for _, freezeID := range freezeIDs {
		if _, err := tx.Exec(
			`INSERT OR IGNORE INTO freeze_data_compressor(freeze_id, selected) VALUES( ?, 0 )`,
			freezeID,
		); err != nil {
			return err
		}
	}

	return nil
}
